<?php

namespace Drupal\flowbite_sdc\Form;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Flowbite SDC settings form.
 */
class CdnSettingsForm extends ConfigFormBase {

  const SETTINGS = 'flowbite_sdc.cdn_settings';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cdn_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [static::SETTINGS];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $form['css_enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use CDN for CSS'),
      '#default_value' => $config->get('css.enable'),
    ];

    $form['css_url'] = [
      '#type' => 'url',
      '#title' => $this->t('CDN URL for CSS'),
      '#description' => $this->t('The URL for CSS files from Flowbite CDN. Restricted to <i>cdn.jsdelivr.net</i> domain.'),
      '#default_value' => $config->get('css.url'),
      '#states' => [
        'visible' => [
          ':input[name="css_enable"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="css_enable"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['js_enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use CDN for JS'),
      '#default_value' => $config->get('js.enable'),
    ];

    $form['js_url'] = [
      '#type' => 'url',
      '#title' => $this->t('CDN URL for JS'),
      '#description' => $this->t('The URL for JS files from Flowbite CDN. Restricted to <i>cdn.jsdelivr.net</i> domain.'),
      '#default_value' => $config->get('js.url'),
      '#states' => [
        'visible' => [
          ':input[name="js_enable"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="js_enable"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue('css_enable') && !$form_state->getValue('css_url')) {
      $form_state->setErrorByName('css_url', $this->t('CDN URL for CSS is required.'));
    }
    elseif ($form_state->getValue('css_enable') && !UrlHelper::isValid($form_state->getValue('css_url'))) {
      $form_state->setErrorByName('css_url', $this->t('CDN URL for CSS is invalid.'));
    }
    elseif ($form_state->getValue('css_enable') && !preg_match('/^https:\/\/cdn\.jsdelivr\.net\/npm\/flowbite@[\d.]+\/.*\.css$/', $form_state->getValue('css_url'))) {
      $form_state->setErrorByName('css_url', $this->t('CDN URL for CSS is invalid and should be from jsdelivr.net domain.'));
    }

    if ($form_state->getValue('js_enable') && !$form_state->getValue('js_url')) {
      $form_state->setErrorByName('js_url', $this->t('CDN URL for JS is required.'));
    }
    elseif ($form_state->getValue('js_enable') && !UrlHelper::isValid($form_state->getValue('js_url'))) {
      $form_state->setErrorByName('js_url', $this->t('CDN URL for JS is invalid.'));
    }
    elseif ($form_state->getValue('js_enable') && !preg_match('/^https:\/\/cdn\.jsdelivr\.net\/npm\/flowbite@[\d.]+\/.*\.js$/', $form_state->getValue('js_url'))) {
      $form_state->setErrorByName('js_url', $this->t('CDN URL for JS is invalid and should be from jsdelivr.net domain.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config(self::SETTINGS)
      ->set('css.enable', $form_state->getValue('css_enable'))
      ->set('css.url', $form_state->getValue('css_enable') ? $form_state->getValue('css_url') : FALSE)
      ->set('js.enable', $form_state->getValue('js_enable'))
      ->set('js.url', $form_state->getValue('js_enable') ? $form_state->getValue('js_url') : FALSE)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
